@@ -1,7 +1,7 @@
 use std::task::{Context, Poll};
 use std::{error::Error, future::Future, marker::PhantomData, pin::Pin};
 
-use ntex_io::{utils::BoxedFactory, Filter, FilterFactory, Io};
+use ntex_io::{Filter, FilterFactory, Io};
 use ntex_service::{Service, ServiceFactory};
 use ntex_util::{future::Ready, time::Millis};
 use tls_openssl::ssl::SslAcceptor;
@@ -35,10 +35,6 @@ impl<F> Acceptor<F> {
         self.acceptor.timeout(timeout);
         self
     }
-
-    pub fn seal(self) -> BoxedFactory<Acceptor<F>, Io<F>> {
-        BoxedFactory::new(self)
-    }
 }
 
 impl<F> From<SslAcceptor> for Acceptor<F> {
