@@ -1,12 +1,12 @@
 #![allow(clippy::type_complexity)]
 //! An implementation of SSL streams for ntex backed by OpenSSL
-use std::sync::Arc;
-use std::{any, future::Future, io, pin::Pin, task::Context, task::Poll};
+use std::{any, cmp, future::Future, io, pin::Pin, task::Context, task::Poll};
+use std::{cell::Cell, sync::Arc};
 
-use ntex_bytes::BytesMut;
+use ntex_bytes::{BytesVec, PoolRef};
 use ntex_io::{Base, Filter, FilterFactory, Io, IoRef, ReadStatus, WriteStatus};
 use ntex_util::time::Millis;
-use tls_rust::{ClientConfig, ServerConfig, ServerName};
+use tls_rust::{Certificate, ClientConfig, ServerConfig, ServerName};
 
 mod accept;
 mod client;
@@ -16,6 +16,14 @@ pub use accept::{Acceptor, AcceptorService};
 use self::client::TlsClientFilter;
 use self::server::TlsServerFilter;
 
+/// Connection's peer cert
+#[derive(Debug)]
+pub struct PeerCert(pub Certificate);
+
+/// Connection's peer cert chain
+#[derive(Debug)]
+pub struct PeerCertChain(pub Vec<Certificate>);
+
 /// An implementation of SSL streams
 pub struct TlsFilter<F = Base> {
     inner: InnerTlsFilter<F>,
@@ -85,7 +93,7 @@ impl<F: Filter> Filter for TlsFilter<F> {
     }
 
     #[inline]
-    fn get_read_buf(&self) -> Option<BytesMut> {
+    fn get_read_buf(&self) -> Option<BytesVec> {
         match self.inner {
             InnerTlsFilter::Server(ref f) => f.get_read_buf(),
             InnerTlsFilter::Client(ref f) => f.get_read_buf(),
@@ -93,7 +101,7 @@ impl<F: Filter> Filter for TlsFilter<F> {
     }
 
     #[inline]
-    fn get_write_buf(&self) -> Option<BytesMut> {
+    fn get_write_buf(&self) -> Option<BytesVec> {
         match self.inner {
             InnerTlsFilter::Server(ref f) => f.get_write_buf(),
             InnerTlsFilter::Client(ref f) => f.get_write_buf(),
@@ -101,21 +109,23 @@ impl<F: Filter> Filter for TlsFilter<F> {
     }
 
     #[inline]
-    fn release_read_buf(
-        &self,
-        io: &IoRef,
-        src: BytesMut,
-        dst: &mut Option<BytesMut>,
-        nb: usize,
-    ) -> io::Result<usize> {
+    fn release_read_buf(&self, buf: BytesVec) {
         match self.inner {
-            InnerTlsFilter::Server(ref f) => f.release_read_buf(io, src, dst, nb),
-            InnerTlsFilter::Client(ref f) => f.release_read_buf(io, src, dst, nb),
+            InnerTlsFilter::Server(ref f) => f.release_read_buf(buf),
+            InnerTlsFilter::Client(ref f) => f.release_read_buf(buf),
         }
     }
 
     #[inline]
-    fn release_write_buf(&self, src: BytesMut) -> Result<(), io::Error> {
+    fn process_read_buf(&self, io: &IoRef, nb: usize) -> io::Result<(usize, usize)> {
+        match self.inner {
+            InnerTlsFilter::Server(ref f) => f.process_read_buf(io, nb),
+            InnerTlsFilter::Client(ref f) => f.process_read_buf(io, nb),
+        }
+    }
+
+    #[inline]
+    fn release_write_buf(&self, src: BytesVec) -> Result<(), io::Error> {
         match self.inner {
             InnerTlsFilter::Server(ref f) => f.release_write_buf(src),
             InnerTlsFilter::Client(ref f) => f.release_write_buf(src),
@@ -229,3 +239,49 @@ impl<F: Filter> FilterFactory<F> for TlsConnectorConfigured {
         Box::pin(async move { TlsClientFilter::create(st, cfg, server_name).await })
     }
 }
+
+pub(crate) struct IoInner<F> {
+    filter: F,
+    pool: PoolRef,
+    read_buf: Cell<Option<BytesVec>>,
+    write_buf: Cell<Option<BytesVec>>,
+    handshake: Cell<bool>,
+}
+
+pub(crate) struct Wrapper<'a, F>(&'a IoInner<F>);
+
+impl<'a, F: Filter> io::Read for Wrapper<'a, F> {
+    fn read(&mut self, dst: &mut [u8]) -> io::Result<usize> {
+        if let Some(mut read_buf) = self.0.filter.get_read_buf() {
+            let len = cmp::min(read_buf.len(), dst.len());
+            let result = if len > 0 {
+                dst[..len].copy_from_slice(&read_buf.split_to(len));
+                Ok(len)
+            } else {
+                Err(io::Error::new(io::ErrorKind::WouldBlock, ""))
+            };
+            self.0.filter.release_read_buf(read_buf);
+            result
+        } else {
+            Err(io::Error::new(io::ErrorKind::WouldBlock, ""))
+        }
+    }
+}
+
+impl<'a, F: Filter> io::Write for Wrapper<'a, F> {
+    fn write(&mut self, src: &[u8]) -> io::Result<usize> {
+        let mut buf = if let Some(mut buf) = self.0.filter.get_write_buf() {
+            buf.reserve(src.len());
+            buf
+        } else {
+            BytesVec::with_capacity_in(src.len(), self.0.pool)
+        };
+        buf.extend_from_slice(src);
+        self.0.filter.release_write_buf(buf)?;
+        Ok(src.len())
+    }
+
+    fn flush(&mut self) -> io::Result<()> {
+        Ok(())
+    }
+}
